// boots & cats &&& | algorithmic break beat generator by Kurt James Werner | fall 2011
// ---------------------------------------------------------------------

import oscP5.*; // for OSC functionality
OscP5  oscP5;   // OSC listener
 
// globals
int zoom = 22;
int x_off = 1; // x and y offsets for the universe display
int y_off = 5;
int beat_off_x = 1;
int beat_off_y = 38;
int boot_off_x = 11;
int boot_off_y = 38;
int pat_off_x = 17;
int pat_off_y = 38;

PFont fontA;

String beat_list = "  one e&a   two e&a three e&a  four e&a"; // beat indicator
String[] beat_list_2 = {"boots", "  &  ", "cats ", " &&& "}; // beat indicator

// setup routine
void setup()
{
  fontA = loadFont("CourierNew36.vlw"); textAlign(LEFT, TOP); // font and text alignment
  textFont(fontA, zoom); // Set the font and its size (in units of pixels)
  
  background(0);
  size((34)*zoom, (32+2+5+4)*zoom);
  oscP5 = new OscP5(this, 8888);
  
  for(int i = 0; i < 33; i++) { fill(255); text("  ////// boots & cats &&& \\\\\\\\\\\\ ".charAt(i), i * zoom, 1 * zoom); }
  for(int i = 0; i < 33; i++) { fill(255); text(      " algorithmic break-beat generator".charAt(i), i * zoom, 2 * zoom); }
  for(int i = 0; i < 33; i++) { fill(255); text(     " by Kurt James Werner / Fall 2011 ".charAt(i), i * zoom, 3 * zoom); }
  
  draw_border(-1,  -5, 32, -1); draw_border(-1,  -1, 32, 32); draw_border(-1,  32, 9,  37); draw_border( 9,  32, 15, 37); draw_border( 15, 32, 32, 37); // borders
  
  noLoop();
}

void draw() { // nothing happpens in the draw loop, except drawing
}

void draw_cells(int[] bitshift_universe) { // draw all of the cells in the universe
  for(int x = 0; x < 32; x++) {
    for(int y = 0; y < 32; y++) {
      fill(0); rect((x + x_off) * zoom, (y + y_off) * zoom, zoom, zoom);
      if( ((bitshift_universe[y] >> x) & 1) == 1) {
        if( random(1) > .5) { 
          fill(255); text("0", (x + x_off) * zoom, (y + y_off) * zoom);
        } else {
          fill(255); text("1", (x + x_off) * zoom, (y + y_off) * zoom);
        }
      }
    }
  }
}

void draw_pat(int pat_x, int pat_y, int pat_value) { // draw a cell of the step sequencer display
  if (pat_y == 3) {fill(0); rect((pat_x+pat_off_x) * zoom, (0 + pat_off_y) * zoom, zoom, zoom); }
  if (pat_y == 1) {fill(0); rect((pat_x+pat_off_x) * zoom, (1 + pat_off_y) * zoom, zoom, zoom); }
  if (pat_y == 2) {fill(0); rect((pat_x+pat_off_x) * zoom, (2 + pat_off_y) * zoom, zoom, zoom); }
  if (pat_y == 0) {fill(0); rect((pat_x+pat_off_x) * zoom, (3 + pat_off_y) * zoom, zoom, zoom); }  
  
  if (pat_value == 1) {
    if (pat_y == 3) {fill(255); text("b", (pat_x+pat_off_x) * zoom, (0 + pat_off_y) * zoom); }
    if (pat_y == 1) {fill(255); text("&", (pat_x+pat_off_x) * zoom, (1 + pat_off_y) * zoom); }
    if (pat_y == 2) {fill(255); text("c", (pat_x+pat_off_x) * zoom, (2 + pat_off_y) * zoom); }
    if (pat_y == 0) {fill(255); text("&", (pat_x+pat_off_x) * zoom, (3 + pat_off_y) * zoom); }
  }
}

void draw_border(int x_start, int y_start, int x_end, int y_end) { // draw a border around a specified area
  for(int x=x_start; x<x_end+1; x++) {
    fill(127); text(char(8212), (x+x_off) * zoom, (y_start +y_off) * zoom);  
    fill(127); text(char(8212), (x+x_off) * zoom, (y_end   +y_off) * zoom);  
  }
  for(int y=y_start; y<y_end+1; y++) {
    fill(127); text(char(124), (x_start+x_off) * zoom, (y+y_off) * zoom);  
    fill(127); text(char(124), (x_end  +x_off) * zoom, (y+y_off) * zoom);  
  }
}

void draw_vertical(int x_start, int y_start, int y_end) { // draw a vertical line (step sequencer)
    for(int y=y_start; y<y_end; y++) {
       fill(63); text("|", (x_start+x_off) * zoom, (y+y_off) * zoom);  
    }
}

void draw_beat(int start_char, int end_char, int what_line) { // draw the beat text
  for(int i = start_char; i < end_char; i++) {
      fill(255); text(beat_list.charAt(i), (i + beat_off_x - what_line * 10) * zoom, (beat_off_y + what_line) * zoom);  
  }
}

void draw_hit(int what_beat) { // draw the instrument text
  for(int i = 0; i < 5; i++) {
      fill(255); text(beat_list_2[what_beat].charAt(i), (i + boot_off_x) * zoom, (boot_off_y + what_beat) * zoom);  
  }
}

void erase_hit(int what_beat) { // erase the instrument text
   for(int i = 0; i < 5; i++) { 
     fill(0); rect((i + boot_off_x) * zoom, boot_off_y * zoom, zoom, 4 * zoom);
   }  
}

// OSC message parser
void oscEvent( OscMessage m ) {
  if ( m.checkAddrPattern( "/draw_cells" ) == true ) { 
    int[] bitshift_universe = new int[32]; 
    if ( m.checkTypetag( "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii" )) { // check typetags
      for(int i = 0; i < 32; i++) {
        bitshift_universe[i] = m.get(i).intValue();
      }
      draw_cells(bitshift_universe);
    }
  }
  
  if ( m.checkAddrPattern( "/draw_border" ) == true ) {
    if ( m.checkTypetag( "iiii" )) { // check typetags
      draw_border(m.get(0).intValue() - 1, m.get(1).intValue() - 1, m.get(2).intValue(), m.get(3).intValue() );
    }
  }
  
  if ( m.checkAddrPattern( "/draw_vertical" ) == true ) {
    if ( m.checkTypetag( "iii" )) { // check typetags
      draw_vertical(m.get(0).intValue(), m.get(1).intValue(), m.get(2).intValue());  
    }
  }
  
  if ( m.checkAddrPattern( "/draw_beat" ) == true ) {
    int beat = 0;
    if ( m.checkTypetag( "i" )) { // check typetags
      beat = m.get(0).intValue() % 16; // get values
    }
    if (beat == 0) {
      for(int j = 0; j < 4; j++) {
        for(int i = 0; i < 9; i++) {
          fill(0); rect((i + beat_off_x) * zoom, (j + beat_off_y) * zoom, zoom, zoom);  
        }
      }  
                       fill(255); draw_beat(0,  6,  0); }
    if (beat == 1 )  { fill(255); draw_beat(6,  7,  0); }
    if (beat == 2 )  { fill(255); draw_beat(7,  8,  0); }
    if (beat == 3 )  { fill(255); draw_beat(8,  9, 0); }
    if (beat == 4 )  { fill(255); draw_beat(9,  16, 1); }
    if (beat == 5 )  { fill(255); draw_beat(16, 17, 1); }
    if (beat == 6 )  { fill(255); draw_beat(17, 18, 1); }
    if (beat == 7 )  { fill(255); draw_beat(18, 19, 1); }
    if (beat == 8 )  { fill(255); draw_beat(19, 26, 2); }
    if (beat == 9 )  { fill(255); draw_beat(26, 27, 2); }
    if (beat == 10 ) { fill(255); draw_beat(27, 28, 2); }
    if (beat == 11 ) { fill(255); draw_beat(28, 29, 2); }
    if (beat == 12 ) { fill(255); draw_beat(29, 36, 3); }
    if (beat == 13 ) { fill(255); draw_beat(36, 37, 3); }
    if (beat == 14 ) { fill(255); draw_beat(37, 38, 3); }
    if (beat == 15 ) { fill(255); draw_beat(38, 39, 3); }
  }
    
  if ( m.checkAddrPattern( "/draw_hit" ) == true ) {
    if ( m.checkTypetag( "i" )) { // check typetags
        draw_hit(m.get(0).intValue()); // get values
    }  
  }    
  
  if ( m.checkAddrPattern( "/erase_hit" ) == true ) {
    if ( m.checkTypetag( "i" )) { // check typetags
        erase_hit(m.get(0).intValue()); // get values
    }  
  }
  if ( m.checkAddrPattern( "/send_pat" ) == true ) {
    if ( m.checkTypetag( "iii" )) { // check typetags
        draw_pat(m.get(0).intValue() % 16, m.get(1).intValue(), m.get(2).intValue()); // get values
    }  
  }
  
  redraw(); // redraw all of the updates to the screen
}  

